--*******************************************************************************************************************************
--
-- Description: This module acquires data and over-range status from a variety of A/D converter(ADC) chips.  The bus type
--				supports multiple RX channels and stuffing of multiple samples per clock cyle to accomodate DDR inputs.  When
--				samples are stuffed, they are chronologically ordered from LSB to MSB.
--					
-- Rev History: 09-04-2013 Initial Release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

Library UNISIM; 
use UNISIM.Vcomponents.all;

use work.model_specific_package.all;
use work.sigfpga_package.all;

entity adc_interface is 
	port (
	--  Reset and clocks supplied by the application logic.
	rst         	: in std_logic;		-- Active high asynchronous reset.
	clk 			: in std_logic;		-- Bus clock.
	rx_clk		: in std_logic;		-- Clock used to register ADC sample data.
	clk200 		: in std_logic;		-- 6.25 MHz clock on global clock buffer.
	
	-- ADC data supplied to the application logic.
	rx_data_out		: out RX_BUS;		-- Data bus carrying ADC samples.
	rx_ovr_out		: out OVR_BIT;		-- Logic one indicates an over-range condition on the ADC.

	-- Serial clock lock command signal supplied from/to the application logic.
	clk_lck_start	: in std_logic; 	-- Single period enable to initiate the clock lock process.
	clk_lck_done	: out std_logic; 	-- Single period flag to indicate that the clock lock process is complete.
	
	--	The following signals connect to ADC devices through FPGA pins.
	rx_data_in		: in RX_BUS_IN;		-- Positive side of the Channel A LVDS ADC data bus.
	rx_ovr_in		: in RX_1;			-- Positive side of the Channel A LVDS ADC over-range flag.
	rx_clk_in		: in RX_1			-- ositive side of the Channel A LVDS ADC output clock.
);	
end adc_interface;

architecture behavioral of adc_interface is

	signal rx_data_dly	: RX_BUS;	-- Time aligned data bus from the IDELAY primitive.
	signal rx_data_re	: RX_BUS;	-- Data bus carrying ADC samples (DDR rising edge).
	signal rx_data_fe	: RX_BUS;	-- Data bus carrying ADC samples (DDR falling edge).
	signal rx_ovr_dly	: RX_1;		-- Time aligned over-range bit from the IDELAY primitive.
	signal rx_ovr_re	: RX_1;		-- Logic one indicates an over-range condition on the ADC (DDR rising edge).
	signal rx_ovr_fe	: RX_1;		-- Logic one indicates an over-range condition on the ADC (DDR falling edge).
	signal rx_clk_dly	: RX_1;		-- Time aligned data ready bit from the IDELAY primitive.
	signal adc_stuffed	: RX_BUS;	-- Potentially multiple ADC samples stuffed on a single bus.
	signal ovr_stuffed	: OVR_BIT;	-- Potentially multiple over range stuffed on a single bus.
	signal rx_clk_re	: std_logic_vector(ADC_CLK_SOURCES-1 downto 0);	-- Registered ADC forwarded clock signal (DDR rising edge).
	signal rx_clk_fe	: std_logic_vector(ADC_CLK_SOURCES-1 downto 0);	-- Registered ADC forwarded clock signal (DDR falling edge).
	signal idly_rst		: std_logic;									-- Logic one enables the IODELAY reset function.
	signal idly_en		: std_logic_vector(ADC_CLK_SOURCES-1 downto 0);	-- Logic one enables the IODELAY increment/decrement function.
	signal inc_dec		: std_logic_vector(ADC_CLK_SOURCES-1 downto 0);	-- Logic one increments the IODELAY one tap and logic zero decrements one tap.
	
begin
 	
--************************************************ INTERNAL SIGNAL ASSIGNMENTS *************************************************

	-- Sequence through each channel.
	ADC_CH : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate
		-- Sequence through each data item bit that gets created in a single clock cycle.
		ADC_BIT : for j in 0 to (RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 generate
			-- There is always new data on the rising clock edge whether data is stuffed (DDR inputs) or not (SDR inputs).
			adc_stuffed(i)(j*RX_INTERLEAVE) <= rx_data_re(i)(j);
			-- If bits are interleaved (DDR), there is also new data on the falling clock edge.
			BIT_INTERLEAVE: if RX_INTERLEAVE > 1 generate
				adc_stuffed(i)((j*RX_INTERLEAVE)+1) <= rx_data_fe(i)(j);
			end generate;
			-- If items are stuffed (DDR), there is also new data on the falling clock edge.
			ITEM_STUFF: if RX_ITEM_STUFF > 1 generate
				adc_stuffed(i)(j+(RX_ITEM_PER_CLK*RX_ITEM_SIZE)) <= rx_data_fe(i)(j);
			end generate;
		end generate;
	end generate;
	
	-- Stuff the over-range bits if necessary.
	OVR_CH : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate
		-- Sequence through each data item that gets created in a single clock cycle.
		OVR_BIT : for j in 0 to RX_ITEM_PER_CLK-1 generate
			-- There is always a new over-range flag on the rising clock edge whether data is stuffed (DDR inputs) or not (SDR inputs).
			-- Note: Replicate a single over-range bit across multiple channels if it is shared.
			ovr_stuffed(i)(j) <= rx_ovr_re((integer((i-1)/RX_CHAN_PER_OVR)*RX_CHAN_PER_OVR)+1);
			-- If items are stuffed (DDR), there is also a new over-range flag on the falling clock edge.
			OVR_STUFF: if RX_ITEM_STUFF > 1 generate
				-- Note: Replicate a single over-range bit across multiple channels if it is shared.
				ovr_stuffed(i)(j+RX_ITEM_PER_CLK) <= rx_ovr_fe((integer((i-1)/RX_CHAN_PER_OVR)*RX_CHAN_PER_OVR)+1);
			end generate;
		end generate;
	end generate;

	-- Register the data and over-range bits to improve timing performance.
	process (rx_clk)
	begin
		if rising_edge(rx_clk) then
			rx_data_out <= adc_stuffed;
			rx_ovr_out <= ovr_stuffed;
		end if;
	end process;
	
--*************************************************** CLOCK LOCK CONTROLLER ****************************************************

    -- Clock lock module to optimize timing at the ADC interface.
    ADC_CLK_LCK : entity work.clock_lock
    generic map (
		SOURCES			=> ADC_CLK_SOURCES,
		MAX_SEARCH		=> ADC_MAX_SEARCH,
		MAX_SEPARATION	=> ADC_MAX_SEPARATION,
		RESET_IF_NOSOL	=> ADC_RESET_IF_NOSOL,
		FINAL_OFFSET	=> ADC_FINAL_OFFSET
	)
	port map (
		rst         	=> rst,
		lock_start   	=> clk_lck_start,
		lock_busy   	=> open,
		lock_done   	=> clk_lck_done,
		lock_status    	=> open,
		clk_lck_clk		=> clk200,
		clk_fwd			=> rx_clk_re,
		clk_lck_edge	=> ADC_CLK_EDGE,
		idly_rst		=> idly_rst,
		idly_en			=> idly_en,
		inc_dec			=> inc_dec
	);
 	
--************************************************** EXTERNAL PIN ASSIGNMENTS **************************************************

	ADC_CHANNEL : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate

		-- Process each ADC data item bit.	
		ADC_DATA : for j in 0 to (RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 generate
			-- Set the IDELAY values for each data bit using the clock lock controller.
			DATA_IDELAY : IDELAY
				generic map (IOBDELAY_TYPE  => "VARIABLE", IOBDELAY_VALUE => ADC_DATA_INIT(((i-1)*RX_ITEM_SIZE/RX_INTERLEAVE)+j))
				port map (I => rx_data_in(i)(j), O => rx_data_dly(i)(j), C => clk200, CE => idly_en(integer((i-1)*ADC_CLK_SOURCES/zero_to_one(RX_CHANNEL_COUNT))), INC => inc_dec(integer((i-1)*ADC_CLK_SOURCES/zero_to_one(RX_CHANNEL_COUNT))), RST => idly_rst);
			-- Use double data rate (DDR) input registers in the IOB to capture data on the rising and falling edge of the sample clock.
			-- Note: The DDR register is used even on SDR inputs to ensure that the signal is registered inside the IOB.
			DATA_DDR : IDDR
				GENERIC MAP(DDR_CLK_EDGE => "SAME_EDGE_PIPELINED") 
				PORT MAP(Q1 => rx_data_re(i)(j), Q2 =>rx_data_fe(i)(j), C => rx_clk, CE => '1', D => rx_data_dly(i)(j), R => rst, S => '0');
		end generate;

		--  Set the IDELAY value for the over-range signals using the clock lock controller. 
		OVR_IDELAY : IDELAY
			generic map (IOBDELAY_TYPE  => "VARIABLE", IOBDELAY_VALUE => ADC_OVR_INIT)
			port map (I => rx_ovr_in(i), O => rx_ovr_dly(i), C => clk200, CE => idly_en(integer((i-1)*ADC_CLK_SOURCES/zero_to_one(RX_CHANNEL_COUNT))), INC => inc_dec(integer((i-1)*ADC_CLK_SOURCES/zero_to_one(RX_CHANNEL_COUNT))), RST => idly_rst);
		-- Use double data rate (DDR) input registers in the IOB to capture data on the rising and falling edge of the sample clock.
		-- Note: The DDR register is used on SDR inputs to ensure that the signal is registered inside the IOB.
		OVR_DDR : IDDR
			GENERIC MAP(DDR_CLK_EDGE => "SAME_EDGE_PIPELINED") 
			PORT MAP(Q1 => rx_ovr_re(i), Q2 => rx_ovr_fe(i), C => rx_clk, CE => '1', D => rx_ovr_dly(i), R => rst, S => '0');
				
	end generate;
	
	-- Process each clock forwarded by the ADC.	
	CLK_FORWARD : for i in 1 to ADC_CLK_SOURCES generate
		--  Set the IDELAY value for the clock forward signals using the clock lock controller.
		CLK_IDELAY : IDELAY
			generic map (IOBDELAY_TYPE => "VARIABLE", IOBDELAY_VALUE => ADC_CLK_INIT)
			port map (I => rx_clk_in(i), O => rx_clk_dly(i), C => clk200, CE => idly_en(integer((i-1)*ADC_CLK_SOURCES/zero_to_one(RX_CHANNEL_COUNT))), INC => inc_dec(integer((i-1)*ADC_CLK_SOURCES/zero_to_one(RX_CHANNEL_COUNT))), RST => idly_rst);
		-- Use double data rate (DDR) input registers in the IOB to capture data on the rising and falling edge of the sample clock.
		-- Note: The DDR register is used on SDR inputs to ensure that the signal is registered inside the IOB.
		CLK_DDR : IDDR
			GENERIC MAP(DDR_CLK_EDGE => "SAME_EDGE_PIPELINED") 
			PORT MAP(Q1 => rx_clk_re(i-1), Q2 => rx_clk_fe(i-1), C => rx_clk, CE => '1', D => rx_clk_dly(i), R => rst, S => '0');
	end generate;
		
end behavioral;
