--*******************************************************************************************************************************
--
-- Description: This module sends an interrupt request to the PLDA EZDMA controller if specific error or status conditions are met.
--					
-- Rev History: 09-04-2013: Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.model_specific_package.all;
use work.sigfpga_package.all;

entity interrupt_controller is
port (
    -- Reset and clocks.
	rst 				: in std_logic;						-- Active high asynchronous reset.
	clk 				: in std_logic;						-- PCI system clock from the Xilinx endpoint core.
	rx_clk				: in std_logic;						-- Clock assigned to the receiver datapath.
	tx_clk				: in std_logic;						-- Clock assigned to the transmitter datapath.

	-- Interrupt request input to the PLDA EZDMA controller.
	int_request 		: out std_logic;					-- Refer to the PLDA EZDMA reference manual.
	
	-- Error status signals.
	fault_status 		: in std_logic_vector(7 downto 0);  -- Bus used to convey fault status since last queried.
	fault_int_mask 		: in std_logic_vector(7 downto 0);	-- Mask bits to suppress interrupts based on fault status updates.
	fault_monitor 		: out std_logic_vector(7 downto 0); -- Bus used to monitor FIFO overflow or underlow errors.
	channel_status 		: in CH_TYPE_3;  					-- Bus used to convey channel status since last queried.
	channel_int_mask 	: in CH_TYPE_3;  					-- Mask bits to suppress interrupts based on channel status updates.
	channel_monitor 	: out CH_TYPE_3; 					-- Bus used to convey DMA refresh status and channel errors.
	bar0_address_error 	: in std_logic_vector(1 downto 0);	-- Bus used to convey a BAR0 PCI read or write error.
	bar2_address_error 	: in std_logic_vector(1 downto 0);	-- Bus used to convey a BAR2 PCI read or write error.
	fifo_error 			: in CH_TYPE_1;						-- Bus used to convey FIFO overflow or underlow errors. 
	rx_tx_error 		: in CH_TYPE_1;						-- Bus used to convey ADC or DAC error condition.  
	clock_status		: in std_logic_vector(1 downto 0);	-- Bus used to report clock stability status.
	
	-- DMA refresh status signals.
	dma_marker	: in CH_TYPE_1;								-- Logic one indicates that the DMA channel has been refreshed.
	
	-- Consolidated interrupt status signals.
	global_int_mask 	: in std_logic;						-- Global interrupt mask to enable/disable all interrupts.
	global_status		: out std_logic_vector(4 downto 0)	-- Consolidated interrupt status bus.
);
end interrupt_controller;

architecture structural of interrupt_controller is

	signal fault_int			: std_logic;				-- Fault interrupt based on status and mask settings.
	signal channel_int			: unsigned(TX_CHANNEL_COUNT+RX_CHANNEL_COUNT-1 downto 0);	-- Channel interrupt vector based on status and mask settings.
	signal rx_tx_error_rclk	: CH_TYPE_1;					-- Single period ADC/DAC error reclocked in the PCI clock domain.
	signal fifo_error_rclk		: CH_TYPE_1;				-- Single period FIFO error reclocked in the PCI clock domain.
	
begin

--***************************************************** RX CHANNEL ERRORS ******************************************************

	RX_CH: if RX_CHANNEL_COUNT > 0 generate
		RX_ERR : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate
			-- Generate a single period ADC error flag in the PCI clock domain.
			ADC_ERR : entity work.clock_crossing
			port map (
				source_clk			=> rx_clk,
				target_clk  		=> clk,
				rst					=> rst,
				source_to_target	=> rx_tx_error(i),
				target_enable		=> rx_tx_error_rclk(i)
			);
			-- Generate a single period FIFO error flag in the PCI clock domain.
			FIFO_ERR : entity work.clock_crossing
			port map (
				source_clk			=> rx_clk,
				target_clk  		=> clk,
				rst					=> rst,
				source_to_target	=> fifo_error(i),
				target_enable		=> fifo_error_rclk(i)
			);
		end generate;
	end generate;
	
--***************************************************** TX CHANNEL ERRORS ******************************************************

	TX_CH: if TX_CHANNEL_COUNT > 0 generate
		TX_ERR : for i in RX_CHANNEL_COUNT+1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
			-- Generate a single period DAC error flag in the PCI clock domain.
			DAC_ERR : entity work.clock_crossing
			port map (
				source_clk			=> tx_clk,
				target_clk  		=> clk,
				rst					=> rst,
				source_to_target	=> rx_tx_error(i),
				target_enable		=> rx_tx_error_rclk(i)
			);
			-- Generate a single period FIFO error flag in the PCI clock domain.
			FIFO_ERR : entity work.clock_crossing
			port map (
				source_clk			=> tx_clk,
				target_clk  		=> clk,
				rst					=> rst,
				source_to_target	=> fifo_error(i),
				target_enable		=> fifo_error_rclk(i)
			);
		end generate;
	end generate;
	
--***************************************************** PORT ASSIGNMENTS *******************************************************

	-- Assign error conditions to the fault status register in BAR0.
	fault_monitor(1 downto 0) <= bar0_address_error;
	fault_monitor(3 downto 2) <= bar2_address_error;
	fault_monitor(5 downto 4) <= not(clock_status);
	fault_monitor(6) <= '0';
	fault_monitor(7) <= '0';
	-- Request an interrupt if any of the unmasked fault status bits are logic one.
	fault_int <= '0' when to_integer(unsigned(fault_int_mask and fault_status)) = 0 else '1';
	
	-- Record status flags on every channel.
	CH_FLAGS : for i in 1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
		-- Assign error conditions to the channel status registers in BAR2.
		channel_monitor(i)(0) <= dma_marker(i);
		channel_monitor(i)(1) <= rx_tx_error_rclk(i);
		channel_monitor(i)(2) <= fifo_error_rclk(i);
		-- Request an interrupt if any of the unmasked channel status bits are logic one.
		channel_int(i-1) <= '0' when to_integer(unsigned(channel_int_mask(i) and channel_status(i))) = 0 else '1';
	end generate;
	
	-- Generate an interrupt if interrupts are enabled and any of the interrupt request bits are logic one.
	int_request <= '0' when global_int_mask = '0' or (to_integer(channel_int) = 0 and fault_int = '0') else '1';
	
	-- Do not apply a mask to the global status bits because they need to be visible when processing channels without interrupts.
	global_status(0) <='0' when to_integer(unsigned(fault_status)) = 0 else '1';
	CH_STAT : for i in 1 to 4 generate
		CH_ACTIVE: if i < TX_CHANNEL_COUNT+RX_CHANNEL_COUNT+1 generate
			global_status(i) <= '0' when to_integer(unsigned(channel_status(i))) = 0 else '1';
		end generate;	
		-- Make signal assignments to unused interrupt channels.
		CH_UNUSED: if i > TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
			global_status(i) <= '0';
		end generate;	
	end generate;	
	
end structural;
