--*******************************************************************************************************************************
--
-- Description:	This module connects the Xilinx PCI Express Endpoint core to the PLDA EZDMA2 core.  It also includes the PCI
--				transaction decoder that generates slave read/write enable signals for the appropriate BAR spaces.
--				The PCI_LANES constant determines the number of electical lanes in the PCIe bus interface.
--				The AXI_WIDTH constant determines whether slave_dataout is 64-bit or 128-bit aligned.
--					
-- Rev History: 09-04-2013:	Inital release.
--
--*******************************************************************************************************************************


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

Library UNISIM; 
use UNISIM.Vcomponents.all; 

use work.model_specific_package.all;
use work.sigfpga_package.all;

entity pci_express is
port (
	-- PCI Express reset, clock and data signals.
	sys_rst_n		: in std_logic;
    sys_clk			: in std_logic;
    pci_exp_rxp		: in std_logic_vector (PCI_LANES-1 downto 0);
    pci_exp_rxn		: in std_logic_vector (PCI_LANES-1 downto 0);
    pci_exp_txp		: out std_logic_vector (PCI_LANES-1 downto 0);
    pci_exp_txn		: out std_logic_vector (PCI_LANES-1 downto 0);
    
    -- Clock and reset generated by the PLDA EZDMA2 core.
    user_clk		: out std_logic;
    user_reset		: out std_logic;
    
    -- Slave (slv) interface connections ot the PLDA EZDMA2 core.
	slv_addr 		: out std_logic_vector(63 downto 0);
	slv_dataout 	: out std_logic_vector(AXI_WIDTH-1 downto 0);
	slv_bytevalid 	: out std_logic_vector((AXI_WIDTH/8)-1 downto 0);
	bar0_rddata 	: in std_logic_vector(AXI_WIDTH-1 downto 0);
	bar0_write 		: out std_logic;
	bar0_read 		: out std_logic;
	bar2_rddata 	: in std_logic_vector(AXI_WIDTH-1 downto 0);
	bar2_write 		: out std_logic;
	bar2_read 		: out std_logic;

	-- Direct memory access (dma) interface connections between the PLDA EZDMA2 core and the application logic.
	dma_wr 			: out std_logic;
	dma_rd 			: out std_logic;
	dma_rdaddr 		: out std_logic_vector(15 downto 0);
	dma_wrchannel 	: out std_logic_vector(15 downto 0);
	dma_rdchannel 	: out std_logic_vector(15 downto 0);
	dma_regin		: in CH_TYPE_128;
	dma_param		: in CH_TYPE_24;
	dma_control		: in CH_TYPE_6;
	dma_status		: out CH_TYPE_4;
	dma_fifocnt		: in CH_TYPE_13;
	dma_rddata		: in CH_TYPE_AXI;
	dma_wrdata		: out std_logic_vector(AXI_WIDTH-1 downto 0);
	
    -- Interrupt controller signals.
	int_request		: in std_logic;
	int_timeout		: in std_logic_vector(27 downto 0);
	int_response	: out std_logic_vector(27 downto 0)
);
end pci_express;

architecture structural of pci_express is

    -- Signals needed to connect the Xilinx PCI Express Endpoint core to the PLDA EZDMA22 core.
	signal user_clk_out				: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal user_reset_out			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal user_lnk_up				: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal tx_buf_av				: std_logic_vector(5 downto 0);		-- Refer to the Xilinx PCI Express documentation.
	signal tx_err_drop				: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal s_axis_tx_tdata			: std_logic_vector(AXI_WIDTH-1 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal s_axis_tx_tvalid			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal s_axis_tx_tready			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal s_axis_tx_tkeep			: std_logic_vector((AXI_WIDTH/8)-1 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal s_axis_tx_tlast			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal s_axis_tx_tuser			: std_logic_vector(3 downto 0);		-- Refer to the Xilinx PCI Express documentation.
	signal m_axis_rx_tdata			: std_logic_vector(AXI_WIDTH-1 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal m_axis_rx_tvalid			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal m_axis_rx_tready			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal m_axis_rx_tkeep			: std_logic_vector((AXI_WIDTH/8)-1 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal m_axis_rx_tlast			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal m_axis_rx_tuser			: std_logic_vector(21 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal rx_np_ok					: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_status				: std_logic_vector(15 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal cfg_command				: std_logic_vector(15 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal cfg_dstatus				: std_logic_vector(15 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal cfg_dcommand				: std_logic_vector(15 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal cfg_lstatus				: std_logic_vector(15 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal cfg_lcommand				: std_logic_vector(15 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal cfg_pcie_link_state		: std_logic_vector(2 downto 0);		-- Refer to the Xilinx PCI Express documentation.
	signal cfg_err_ecrc				: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_err_ur				: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_err_cpl_timeout		: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_err_cpl_unexpect		: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_err_cpl_abort		: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_err_posted			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_err_cor				: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_err_tlp_cpl_header	: std_logic_vector(47 downto 0);	-- Refer to the Xilinx PCI Express documentation.
	signal cfg_err_cpl_rdy			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_err_locked			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_trn_pending			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_interrupt			: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_interrupt_rdy		: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_interrupt_assert		: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_interrupt_di			: std_logic_vector(7 downto 0);		-- Refer to the Xilinx PCI Express documentation.
	signal cfg_interrupt_do			: std_logic_vector(7 downto 0);		-- Refer to the Xilinx PCI Express documentation.
	signal cfg_interrupt_mmenable	: std_logic_vector(2 downto 0);		-- Refer to the Xilinx PCI Express documentation.
	signal cfg_interrupt_msienable	: std_logic;						-- Refer to the Xilinx PCI Express documentation.
	signal cfg_bus_number			: std_logic_vector(7 downto 0);		-- Refer to the Xilinx PCI Express documentation.
	signal cfg_device_number		: std_logic_vector(4 downto 0);		-- Refer to the Xilinx PCI Express documentation.
	signal cfg_function_number		: std_logic_vector(2 downto 0);		-- Refer to the Xilinx PCI Express documentation.
	signal user_lnk_upn				: std_logic;						-- Refer to the PLDA EZDMA2 reference design.
	signal rstn						: std_logic;						-- Refer to the PLDA EZDMA2 reference design.

	-- Slave (slv) interface connections between the PLDA EZDMA2 core and the transaction decoder.
	signal slv_bytecount 	: std_logic_vector(12 downto 0);			-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_bar 			: std_logic_vector(6 downto 0);				-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_cpladdr 		: std_logic_vector(31 downto 0);			-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_cplparam 	: std_logic_vector(7 downto 0);				-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_writereq		: std_logic;								-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_readreq		: std_logic;								-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_write		: std_logic;								-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_accept		: std_logic;								-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_abort		: std_logic;								-- Refer to the PLDA EZDMA2 reference manual.

	-- Direct memory access (dma) interface connections between the PLDA EZDMA2 core and the transaction decoder.
	signal dma_rddata_mux 	: std_logic_vector(AXI_WIDTH-1 downto 0);	-- Data bus used for DMA write transactions through the PLDA EZDMA2 core.
	signal int_ack			: std_logic;								-- Refer to the PLDA EZDMA2 reference manual.
	signal int_req_throttle	: std_logic;								-- Throttled interrupts to the EZDMA2 core.
	
	-- Signals used local (_local) to this module that are also assigned to ports.
	signal slv_addr_local 		: std_logic_vector(63 downto 0);		-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_rd_local			: std_logic;							-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_rdchannel_local	: std_logic_vector(15 downto 0);		-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_regin_local		: DMA_TYPE_128;							-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_param_local		: DMA_TYPE_24;							-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_control_local	: DMA_TYPE_6;							-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_status_local		: DMA_TYPE_4;							-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_fifocnt_local	: DMA_TYPE_13;							-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_rddata_local		: DMA_TYPE_AXI;							-- Refer to the PLDA EZDMA2 reference manual.
	
	signal test_mode	: std_logic_vector(15 downto 0);		-- Refer to the PLDA EZDMA2 reference manual.
	signal int_msgnum	: std_logic_vector(4 downto 0);		-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_ur			: std_logic;							-- Refer to the PLDA EZDMA2 reference manual.
	
	
begin

    -- Xilinx 8-lane PCI Express Endpoint core.
    EP : entity work.pcie_endpoint
	port map (
		-- Serial bus interface.
		pci_exp_txp									=> pci_exp_txp,
		pci_exp_txn									=> pci_exp_txn,
		pci_exp_rxp									=> pci_exp_rxp,
		pci_exp_rxn									=> pci_exp_rxn,
		-- Partial reconfiguration clocking.		
		pipe_pclk_in								=> '0',
		pipe_rxusrclk_in							=> '0',
		pipe_rxoutclk_in							=> x"00",
		pipe_dclk_in								=> '0',
		pipe_userclk1_in							=> '0',
		pipe_userclk2_in							=> '0',         
		pipe_oobclk_in								=> '0',
		pipe_mmcm_lock_in							=> '0',
		pipe_txoutclk_out							=> open,
		pipe_rxoutclk_out							=> open,
		pipe_pclk_sel_out							=> open,
		pipe_gen3_out								=> open,
		-- Common application interface.
		user_clk_out								=> user_clk_out ,
		user_reset_out								=> user_reset_out,
		user_lnk_up									=> user_lnk_up,
		user_app_rdy								=> open,
		-- Transmit application interface.
		tx_buf_av									=> tx_buf_av ,
		tx_cfg_req									=> open ,
		tx_err_drop									=> tx_err_drop ,
		s_axis_tx_tready							=> s_axis_tx_tready ,
		s_axis_tx_tdata								=> s_axis_tx_tdata ,
		s_axis_tx_tkeep								=> s_axis_tx_tkeep ,
		s_axis_tx_tlast								=> s_axis_tx_tlast ,
		s_axis_tx_tvalid							=> s_axis_tx_tvalid ,
		s_axis_tx_tuser								=> s_axis_tx_tuser,
		tx_cfg_gnt									=> '1' ,
		-- Receive application interface.
		m_axis_rx_tdata								=> m_axis_rx_tdata ,
		m_axis_rx_tkeep								=> m_axis_rx_tkeep ,
		m_axis_rx_tlast								=> m_axis_rx_tlast ,
		m_axis_rx_tvalid							=> m_axis_rx_tvalid ,
		m_axis_rx_tready							=> m_axis_rx_tready ,
		m_axis_rx_tuser								=> m_axis_rx_tuser,
		rx_np_ok									=> rx_np_ok ,
		rx_np_req									=> '1' ,
		-- Flow control status.
		fc_cpld										=> open,
		fc_cplh										=> open,
		fc_npd										=> open,
		fc_nph										=> open,
		fc_pd										=> open,
		fc_ph										=> open,
		fc_sel										=> "000",
		-- Configuration interface (EP and RP).
		cfg_mgmt_do									=> open ,
		cfg_mgmt_rd_wr_done							=> open ,
		cfg_status									=> cfg_status ,
		cfg_command									=> cfg_command ,
		cfg_dstatus									=> cfg_dstatus ,
		cfg_dcommand								=> cfg_dcommand ,
		cfg_lstatus									=> cfg_lstatus ,
		cfg_lcommand								=> cfg_lcommand ,
		cfg_dcommand2								=> open ,
		cfg_pcie_link_state							=> cfg_pcie_link_state ,
		cfg_pmcsr_pme_en							=> open ,
		cfg_pmcsr_powerstate						=> open ,
		cfg_pmcsr_pme_status						=> open ,
		cfg_received_func_lvl_rst					=> open ,
		cfg_mgmt_di									=> x"00000000",
		cfg_mgmt_byte_en							=> x"0",
		cfg_mgmt_dwaddr								=> "0000000000",
		cfg_mgmt_wr_en								=> '0',
		cfg_mgmt_rd_en								=> '0',
		cfg_mgmt_wr_readonly						=> '0',
		cfg_err_ecrc								=> cfg_err_ecrc ,
		cfg_err_ur									=> cfg_err_ur ,
		cfg_err_cpl_timeout							=> cfg_err_cpl_timeout ,
		cfg_err_cpl_unexpect						=> cfg_err_cpl_unexpect ,
		cfg_err_cpl_abort							=> cfg_err_cpl_abort ,
		cfg_err_posted								=> cfg_err_posted ,
		cfg_err_cor									=> cfg_err_cor ,
		cfg_err_atomic_egress_blocked				=> '0' ,
		cfg_err_internal_cor						=> '0' ,
		cfg_err_malformed							=> '0' ,
		cfg_err_mc_blocked							=> '0' ,
		cfg_err_poisoned							=> '0' ,
		cfg_err_norecovery							=> '0' ,
		cfg_err_tlp_cpl_header						=> cfg_err_tlp_cpl_header,
		cfg_err_cpl_rdy								=> cfg_err_cpl_rdy ,
		cfg_err_locked								=> cfg_err_locked ,
		cfg_err_acs									=> '0' ,
		cfg_err_internal_uncor						=> '0' ,
		cfg_trn_pending								=> '0' ,
		cfg_pm_halt_aspm_l0s						=> '0' ,
		cfg_pm_halt_aspm_l1							=> '0' ,
		cfg_pm_force_state_en						=> '0' ,
		cfg_pm_force_state							=> "00" ,
		cfg_dsn										=> (others=>'0') ,
		-- Configuration interface (EP).
		cfg_interrupt								=> cfg_interrupt ,
		cfg_interrupt_rdy							=> cfg_interrupt_rdy ,
		cfg_interrupt_assert						=> cfg_interrupt_assert ,
		cfg_interrupt_di							=> cfg_interrupt_di ,
		cfg_interrupt_do							=> cfg_interrupt_do ,
		cfg_interrupt_mmenable						=> cfg_interrupt_mmenable ,
		cfg_interrupt_msienable						=> cfg_interrupt_msienable ,
		cfg_interrupt_msixenable					=> open,
		cfg_interrupt_msixfm						=> open,
		cfg_interrupt_stat							=> '0',
		cfg_pciecap_interrupt_msgnum				=> "00000",
		cfg_to_turnoff								=> open ,
		cfg_turnoff_ok								=> '0' ,
		cfg_bus_number								=> cfg_bus_number ,
		cfg_device_number							=> cfg_device_number ,
		cfg_function_number							=> cfg_function_number ,
		cfg_pm_wake									=> '0' ,
		-- Configuration interface (RP).
		cfg_msg_received 							=> open ,
		cfg_msg_data								=> open ,
		cfg_msg_received_pm_as_nak					=> open ,
		cfg_msg_received_setslotpowerlimit			=> open ,
		cfg_pm_send_pme_to							=> '0' ,
		cfg_ds_bus_number							=> x"00" ,
		cfg_ds_device_number						=> "00000" ,
		cfg_ds_function_number						=> "000" ,
		cfg_mgmt_wr_rw1c_as_rw						=> '0' ,
		cfg_bridge_serr_en							=> open ,
		cfg_slot_control_electromech_il_ctl_pulse	=> open ,
		cfg_root_control_syserr_corr_err_en			=> open ,
		cfg_root_control_syserr_non_fatal_err_en	=> open ,
		cfg_root_control_syserr_fatal_err_en		=> open ,
		cfg_root_control_pme_int_en					=> open ,
		cfg_aer_rooterr_corr_err_reporting_en		=> open ,
		cfg_aer_rooterr_non_fatal_err_reporting_en	=> open ,
		cfg_aer_rooterr_fatal_err_reporting_en		=> open ,
		cfg_aer_rooterr_corr_err_received			=> open ,
		cfg_aer_rooterr_non_fatal_err_received		=> open ,
		cfg_aer_rooterr_fatal_err_received			=> open ,
		cfg_msg_received_err_cor					=> open ,
		cfg_msg_received_err_non_fatal				=> open ,
		cfg_msg_received_err_fatal					=> open ,
		cfg_msg_received_pm_pme						=> open ,
		cfg_msg_received_pme_to_ack					=> open ,
		cfg_msg_received_assert_int_a				=> open ,
		cfg_msg_received_assert_int_b				=> open ,
		cfg_msg_received_assert_int_c				=> open ,
		cfg_msg_received_assert_int_d				=> open ,
		cfg_msg_received_deassert_int_a				=> open ,
		cfg_msg_received_deassert_int_b				=> open ,
		cfg_msg_received_deassert_int_c				=> open ,
		cfg_msg_received_deassert_int_d				=> open ,
		-- Physical layer control and status (EP and RP).		
		pl_directed_link_change						=> "00",
		pl_directed_link_width						=> "00",
		pl_directed_link_speed						=> '0',
		pl_directed_link_auton						=> '0',
		pl_upstream_prefer_deemph					=> '0',
		pl_sel_lnk_rate								=> open,
		pl_sel_lnk_width							=> open,
		pl_ltssm_state								=> open,
		pl_lane_reversal_mode						=> open,
		pl_phy_lnk_up								=> open,
		pl_tx_pm_state								=> open,
		pl_rx_pm_state								=> open,
		pl_link_upcfg_cap							=> open,
		pl_link_gen2_cap							=> open,
		pl_link_partner_gen2_supported				=> open,                      
		pl_initial_link_width						=> open,
		pl_directed_change_done						=> open ,
		-- Physical layer control and status (EP).		
		pl_received_hot_rst							=> open ,
		-- Physical layer control and status (RP).		
		pl_transmit_hot_rst							=> '0' ,
		pl_downstream_deemph_source					=> '0' ,
		-- Advanced error reporting (AER).
		cfg_err_aer_headerlog						=> (others=>'0'),
		cfg_aer_interrupt_msgnum					=> "00000",
		cfg_err_aer_headerlog_set					=> open,
		cfg_aer_ecrc_check_en						=> open,
		cfg_aer_ecrc_gen_en							=> open,
		-- Virtual channel.		
		cfg_vc_tcvc_map								=> open ,
		-- Dynamic reconfiguration port (DRP).
		pcie_drp_clk								=> '0',
		pcie_drp_en									=> '0',
		pcie_drp_we									=> '0',
		pcie_drp_addr								=> "000000000",
		pcie_drp_di									=> x"0000",
		pcie_drp_rdy								=> open,
		pcie_drp_do									=> open,
		-- Tandem configuration interface.
		startup_cfgclk								=> open,
		startup_cfgmclk								=> open,
		startup_eos									=> open,
		startup_preq								=> open,
		startup_clk									=> '0', 
		startup_gsr									=> '0', 
		startup_gts									=> '0', 
		startup_keyclearb							=> '1', 
		startup_pack								=> '0', 
		startup_usrcclko							=> '0', 
		startup_usrcclkts							=> '1', 
		startup_usrdoneo							=> '0', 
		startup_usrdonets							=> '1', 
		icap_clk									=> '0',
		icap_csib									=> '0',
		icap_rdwrb									=> '0', 
		icap_i										=> x"00000000", 
		icap_o										=> open,
		-- Partial reconfiguration clocking.		
		pipe_mmcm_rst_n								=> '1',
		-- System bus clock and reset.		
		sys_clk										=> sys_clk ,
		sys_rst_n									=> sys_rst_n
	);
          	
    -- Create a legacy active low reset to make the 7 Series endpoint core compatible with the PLDA EZDMA2 core.
    rstn <= not(user_reset_out);
    user_lnk_upn <= not(user_lnk_up);
    
	test_mode <= (others=>'0');
	int_msgnum <= (others=>'0');
	slv_ur <= '0';
	
    -- PLDA EZDMA2 core.
	EZDMA2 : plda_ezdma2
	port map (
		clk                     => user_clk_out,		-- Signal names don't match.
		rstn                    => rstn,				-- Polarity change.
		test_mode               => test_mode,
		user_lnk_up             => user_lnk_upn,		-- Polarity change.
		s_axis_tx_tdata         => s_axis_tx_tdata,
		s_axis_tx_tkeep         => s_axis_tx_tkeep,
		s_axis_tx_tlast         => s_axis_tx_tlast,
		s_axis_tx_tvalid        => s_axis_tx_tvalid,
		s_axis_tx_tready        => s_axis_tx_tready,
		s_axis_tx_tuser         => s_axis_tx_tuser,
		tx_err_drop             => tx_err_drop,
		tbuf_av                 => tx_buf_av,			-- Signal names don't match.
		m_axis_rx_tdata         => m_axis_rx_tdata,
		m_axis_rx_tkeep         => m_axis_rx_tkeep,
		m_axis_rx_tlast         => m_axis_rx_tlast,
		m_axis_rx_tvalid        => m_axis_rx_tvalid,
		m_axis_rx_tuser         => m_axis_rx_tuser,
		m_axis_rx_tready        => m_axis_rx_tready,
		rx_rnp_ok               => rx_np_ok,			-- Signal names don't match.
		cfg_err_cor             => cfg_err_cor,
		cfg_err_cpl_abort       => cfg_err_cpl_abort,
		cfg_err_cpl_timeout     => cfg_err_cpl_timeout,
		cfg_err_cpl_unexpect    => cfg_err_cpl_unexpect,
		cfg_err_ecrc            => cfg_err_ecrc,
		cfg_err_posted          => cfg_err_posted,
		cfg_err_tlp_cpl_header  => cfg_err_tlp_cpl_header,
		cfg_err_ur              => cfg_err_ur,
		cfg_err_cpl_rdy         => cfg_err_cpl_rdy,
		cfg_err_locked          => cfg_err_locked,
		cfg_interrupt           => cfg_interrupt,
		cfg_interrupt_rdy       => cfg_interrupt_rdy,
		cfg_pcie_link_state     => cfg_pcie_link_state,
		cfg_interrupt_assert    => cfg_interrupt_assert,
		cfg_interrupt_di        => cfg_interrupt_di,
		cfg_interrupt_do        => cfg_interrupt_do,
		cfg_interrupt_mmenable  => cfg_interrupt_mmenable,
		cfg_interrupt_msienable => cfg_interrupt_msienable,
		cfg_trn_pending         => cfg_trn_pending,
		cfg_bus_number          => cfg_bus_number,
		cfg_device_number       => cfg_device_number,
		cfg_function_number     => cfg_function_number,
		cfg_status              => cfg_status,
		cfg_command             => cfg_command,
		cfg_dstatus             => cfg_dstatus,
		cfg_dcommand            => cfg_dcommand,
		cfg_lstatus             => cfg_lstatus,
		cfg_lcommand            => cfg_lcommand,
		int_request             => int_req_throttle,
		int_ack                 => int_ack,
		int_msgnum              => int_msgnum,
		cfg_prmcsr              => open,
		cfg_devcsr              => open,
		cfg_linkcsr             => open,
		cfg_msicsr              => open,
		cfg_ltssm               => open,
		slv_dataout             => slv_dataout,
		slv_bytevalid           => slv_bytevalid,
		slv_bytecount           => slv_bytecount,
		slv_dwcount             => open,
		slv_addr                => slv_addr_local,
		slv_bar                 => slv_bar,           
		slv_readreq             => slv_readreq,              
		slv_cpladdr             => slv_cpladdr,
		slv_cplparam            => slv_cplparam,
		slv_writereq            => slv_writereq,
		slv_write               => slv_write,
		slv_lastwrite           => open,
		slv_io                  => open,
		slv_accept              => slv_accept,
		slv_abort               => slv_abort,
		slv_ur                  => slv_ur,           
		dma_rd                  => dma_rd_local,
		dma_rdaddr              => dma_rdaddr,
		dma_rdchannel           => dma_rdchannel_local,
		dma_rddata              => dma_rddata_mux,                               
		dma_wr                  => dma_wr,
		dma_wraddr              => open,
		dma_wrchannel           => dma_wrchannel,
		dma_wrdata              => dma_wrdata,
		dma_wrbytevalid         => open,                
		dma0_regin              => dma_regin_local(0),            
		dma0_regout             => open,
		dma0_param              => dma_param_local(0),
		dma0_control            => dma_control_local(0),
		dma0_status             => dma_status_local(0),
		dma0_fifocnt            => dma_fifocnt_local(0),
		dma1_regin              => dma_regin_local(1),  
		dma1_regout             => open,
		dma1_param              => dma_param_local(1),
		dma1_control            => dma_control_local(1),
		dma1_status             => dma_status_local(1),
		dma1_fifocnt            => dma_fifocnt_local(1),
		dma2_regin              => dma_regin_local(2),
		dma2_regout             => open,
		dma2_param              => dma_param_local(2),
		dma2_control            => dma_control_local(2),
		dma2_status             => dma_status_local(2),
		dma2_fifocnt            => dma_fifocnt_local(2),
		dma3_regin              => dma_regin_local(3),
		dma3_regout             => open,
		dma3_param              => dma_param_local(3),
		dma3_control            => dma_control_local(3),
		dma3_status             => dma_status_local(3),
		dma3_fifocnt            => dma_fifocnt_local(3),
		dma4_regin              => dma_regin_local(4),
		dma4_regout             => open,
		dma4_param              => dma_param_local(4),
		dma4_control            => dma_control_local(4),
		dma4_status             => dma_status_local(4),
		dma4_fifocnt            => dma_fifocnt_local(4),
		dma5_regin              => dma_regin_local(5),
		dma5_regout             => open,
		dma5_param              => dma_param_local(5),
		dma5_control            => dma_control_local(5),
		dma5_status             => dma_status_local(5),
		dma5_fifocnt            => dma_fifocnt_local(5),
		dma6_regin              => dma_regin_local(6),
		dma6_regout             => open,
		dma6_param              => dma_param_local(6),
		dma6_control            => dma_control_local(6),
		dma6_status             => dma_status_local(6),
		dma6_fifocnt            => dma_fifocnt_local(6),
		dma7_regin              => dma_regin_local(7),
		dma7_regout             => open,
		dma7_param              => dma_param_local(7),
		dma7_control            => dma_control_local(7),
		dma7_status             => dma_status_local(7),
		dma7_fifocnt            => dma_fifocnt_local(7)
	);

    -- PCI transaction decoder that interfaces to the PLDA EZDMA2 core.
   	PCI_DECODER : entity work.pci_transaction_decoder
	port map (
		rst => user_reset_out,
		clk => user_clk_out,
	    slv_bar0 => slv_bar(0),
	    slv_bar2 => slv_bar(2),
	    slv_addr => slv_addr_local(15 downto 0),
	    slv_readreq => slv_readreq,
	    slv_writereq => slv_writereq,
	    slv_write => slv_write,
	    slv_bytecount => slv_bytecount,
	    slv_cpladdr => slv_cpladdr,
	    slv_cplparam => slv_cplparam,
	    slv_accept => slv_accept,
	    slv_abort => slv_abort,
	    dma_regin => dma_regin_local(0),
	    dma_param => dma_param_local(0),
	    dma_control => dma_control_local(0),
	    dma_status => dma_status_local(0)(3),
	    dma_fifocnt => dma_fifocnt_local(0),
	    dma_rdchannel => dma_rdchannel_local(7 downto 0),
	    dma_rddata_mux => dma_rddata_mux,
		dma_rd	=> dma_rd_local,
	    bar0_write => bar0_write,
	    bar2_write => bar2_write,
	    bar0_rddata => bar0_rddata,
	    bar2_rddata => bar2_rddata,
		bar0_read => bar0_read,
		bar2_read => bar2_read,
		bar0_read_latency => "00",
		bar2_read_latency => "01",
		dma1_rddata => dma_rddata_local(1),
		dma2_rddata => dma_rddata_local(2),
		dma3_rddata => dma_rddata_local(3),
		dma4_rddata => dma_rddata_local(4),
		dma5_rddata => dma_rddata_local(5),
		dma6_rddata => dma_rddata_local(6),         
		dma7_rddata => dma_rddata_local(7)
	);

    -- Limit the time that an interrupt request to the PLDA EZDMA2 core can remain active.
   	INT_TIMER : entity work.interrupt_timer
	port map (
		rst 				=> user_reset_out,
		clk 				=> user_clk_out,
		int_timeout 		=> int_timeout,
		int_response 		=> int_response,
		int_request 		=> int_request,
		int_ack				=> int_ack,
		int_req_throttle	=> int_req_throttle
	);
	
	-- Assign local signals to ports.
	user_clk		<=	user_clk_out;
	user_reset		<=	user_reset_out;
	slv_addr		<= 	slv_addr_local;
	dma_rd			<=	dma_rd_local;
	dma_rdchannel	<=	dma_rdchannel_local;
	
	-- Make signal assignments to unused DMA channels.
	DMA_CH : for i in 1 to 7 generate
		CH_ACTIVE: if i < TX_CHANNEL_COUNT+RX_CHANNEL_COUNT+1 generate
			dma_regin_local(i) <= dma_regin(i);
			dma_param_local(i) <= dma_param(i);
			dma_control_local(i) <= dma_control(i);
			dma_status(i) <= dma_status_local(i);
			dma_fifocnt_local(i) <= dma_fifocnt(i);
			dma_rddata_local(i) <= dma_rddata(i);
		end generate;	
		CH_UNUSED: if i > TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
			dma_regin_local(i) <= (others=>'0');
			dma_param_local(i) <= (others=>'0');
			dma_control_local(i) <= (others=>'0');
			dma_fifocnt_local(i) <= (others=>'0');
			dma_rddata_local(i) <= (others=>'0');
		end generate;	
	end generate;	
		
end structural;
