--*******************************************************************************************************************************
--
-- Description: Generic transceiver design applied to all SigFPGA products.
--					
-- Rev History: 09-04-2013:	Initial release.
--                                                  
--*******************************************************************************************************************************
                                             
library ieee,unisim;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use unisim.VCOMPONENTS.all;
use work.model_specific_package.all;
use work.sigfpga_package.all;

entity transceiver is
port (
	sys_rst_n		: in std_logic;
    sys_clk			: in std_logic;
    pci_exp_rxp		: in std_logic_vector (PCI_LANES-1 downto 0);
    pci_exp_rxn		: in std_logic_vector (PCI_LANES-1 downto 0);
    pci_exp_txp		: out std_logic_vector (PCI_LANES-1 downto 0);
    pci_exp_txn		: out std_logic_vector (PCI_LANES-1 downto 0);
    clk200_in		: in std_logic;
    sample_clk_in  	: in std_logic;
    rx_data_in  	: in RX_BUS_IN;
    rx_ovr_in		: in RX_1;
    rx_clk_in		: in RX_1;
    tx_data_out  	: out TX_BUS;
    tx_clk_in		: in TX_1;
    tx_clk_out		: out TX_1;
	i2c_scl 		: out std_logic;
	i2c_sda_in 		: in std_logic;
	i2c_sda_out 	: out std_logic;
	i2c_direction 	: out std_logic;
    spi_clk			: out std_logic;
    spi_sdi			: out std_logic;
    spi_sdo			: in  std_logic;
    spi_csb_cmd     : out std_logic_vector(2 downto 0);
    synth_cmd		: out std_logic_vector(2 downto 0);
    led_on			: out std_logic_vector(1 downto 0);
    coax_trig		: in std_logic;
    gpio_trig		: in std_logic;
    gpio_in			: in std_logic_vector(GPIO_COUNT downto 1);
    gpio_out		: out std_logic_vector(GPIO_COUNT downto 1);
    gpio_direction	: out std_logic_vector(GPIO_COUNT downto 1);
    gpio_tristate	: out std_logic;
    user_in			: in std_logic_vector(USER_COUNT downto 1);
    user_out		: out std_logic_vector(USER_COUNT downto 1);
    user_direction	: out std_logic_vector(USER_COUNT downto 1);
    user_tristate	: out std_logic;
    gpio_user_cmd	: out std_logic_vector(2 downto 0)
);                                                                                            
end transceiver;

architecture structural of transceiver is

	-- Clock and reset signals.
	signal user_clk			: std_logic;	-- Clock supplied by Xilinx PCI Express Endpoint core (125 MHz or 250 MHz).
	signal user_reset		: std_logic;	-- Active high reset supplied by PCI Express bus.
 	signal rst				: std_logic;	-- Active high reset derived from PCI Express active low reset.
    signal software_reset	: std_logic;	-- Active high reset tied to a register for software control.
     		
	-- Slave (slv) interface connections between the PLDA EZDMA2 core and the application logic.
	signal slv_dataout 			: std_logic_vector(AXI_WIDTH-1 downto 0);	-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_bytevalid 		: std_logic_vector((AXI_WIDTH/8)-1 downto 0);		-- Refer to the PLDA EZDMA2 reference manual.
	signal slv_addr 			: std_logic_vector(63 downto 0);	-- Refer to the PLDA EZDMA2 reference manual.
	signal bar0_write			: std_logic;	-- Logic one indicates that the current PCI write request is directed at BAR0.
	signal bar2_write			: std_logic;	-- Logic one indicates that the current PCI write request is directed at BAR2.
	signal bar0_read			: std_logic;	-- Logic one indicates that the current PCI read request is directed at BAR0.
	signal bar2_read			: std_logic;	-- Logic one indicates that the current PCI read request is directed at BAR2.
	signal bar0_rddata			: std_logic_vector(AXI_WIDTH-1 downto 0);	-- Data return from read operation on BAR0.
	signal bar2_rddata			: std_logic_vector(AXI_WIDTH-1 downto 0);	-- Data return from read operation on BAR2.

	-- Direct memory access (dma) interface connections between the PLDA EZDMA2 core and the application logic.
	signal dma_wr 				: std_logic;						-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_rd 				: std_logic;						-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_rdaddr 			: std_logic_vector(15 downto 0);	-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_wrchannel 		: std_logic_vector(15 downto 0);	-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_rdchannel 		: std_logic_vector(15 downto 0);	-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_regin			: CH_TYPE_128;						-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_param			: CH_TYPE_24;						-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_control			: CH_TYPE_6;						-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_status			: CH_TYPE_4;						-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_fifocnt			: CH_TYPE_13;						-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_rddata			: CH_TYPE_AXI;						-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_wrdata			: std_logic_vector(AXI_WIDTH-1 downto 0);	-- Refer to the PLDA EZDMA2 reference manual.
	signal dma_page_address		: CH_TYPE_64;						-- Page address stored in the BAR0 memory.
	signal dma_descriptor		: CH_TYPE_32;						-- Page descriptor stored in the BAR0 memory.
	signal dma_current_page		: CH_TYPE_10;						-- Page number of the active DMA buffer.
	signal dma_current_burst	: CH_TYPE_16;						-- Number of the active DMA burst within a page.
	signal dma_marker 			: CH_TYPE_1;						-- Logic one indicates that the DMA channel has been refreshed.
	signal dma_engine			: CH_1TO4_3;						-- Status of DMA engine busy (2), go (1), and enable (0) bits.
	
	-- Serial peripheral interface bus signals.
	signal serial_config		: std_logic_vector(6 downto 0);		-- Serial bus command word.
	signal serial_payload		: std_logic_vector(7 downto 0);		-- Serial bus payload (instruction + data).
	signal serial_start			: std_logic;						-- Logic one initiates a SPI bus transaction.
	signal serial_busy			: std_logic;	-- Logic one indicates that the process is active.
	signal serial_shift			: std_logic; 	-- Logic one indicates that the register needs to perform a serial shift.
	signal serial_read			: std_logic; 	-- Registered duplicate of sdo.

    -- Interrupt controller signals.
	signal int_request			: std_logic;						-- Refer to the PLDA EZDMA2 reference manual.
	signal int_timeout			: std_logic_vector(27 downto 0);	-- Maximum time to hold an interrupt request active.
	signal int_response			: std_logic_vector(27 downto 0);	-- Time taken to service the last interrupt or timeout value.
	signal global_int_mask		: std_logic;						-- Global interrupt mask to enable/disable all interrupts.
	signal global_status		: std_logic_vector(4 downto 0);		-- Consolidated interrupt status bus.  
	signal fault_monitor		: std_logic_vector(7 downto 0);		-- Bus used to convey instantaneous fault conditions.        
    signal fault_status			: std_logic_vector(7 downto 0);		-- Bus used to convey fault status since last queried.
	signal fault_int_mask		: std_logic_vector(7 downto 0);		-- Mask bits to suppress interrupts based on fault status updates.
	signal channel_monitor 		: CH_TYPE_3; 						-- Bus used to convey DMA refresh status and channel errors.
	signal channel_status 		: CH_TYPE_3;  						-- Bus used to convey channel status since last queried.
	signal channel_int_mask 	: CH_TYPE_3;  						-- Mask bits to suppress interrupts based on channel status updates.
	signal fifo_error			: CH_TYPE_1;						-- Bus used to convey FIFO overflow or underlow errors.             
	signal rx_tx_error			: CH_TYPE_1;						-- Bus used to convey ADC or DAC error condition.   
	signal bar0_address_error	: std_logic_vector(1 downto 0);		-- Bus used to convey a BAR0 PCI read or write error.  
	signal bar2_address_error	: std_logic_vector(1 downto 0);		-- Bus used to convey a BAR2 PCI read or write error.     

    -- Command and status signals for each channel.
	signal synchronizer_status	: CH_TYPE_2;	-- Indicator bits for busy (0) and ready (1) status reporting.
	signal synchronizer_on_off	: CH_TYPE_2;	-- Active high on/off (bit 0/bit 1) switch for the synchronizer.
	signal synchronizer			: CH_TYPE_36;	-- Register containing start and stop event selections.
	
    signal tx_data_in			: TX_BUS;		-- Data bus carrying DAC samples to the DAC interface core.
    signal rx_data_out			: RX_BUS;		-- Data bus carrying ADC samples from the ADC interface core.
	signal rx_ovr_out			: OVR_BIT;
	signal clk_init_start		: std_logic;	-- Single period hardware initialization trigger.
	signal clk_init_busy		: std_logic;	-- Hardware initialization state machine status.
	signal soft_trig			: std_logic;	-- Single period software trigger generated in the PCI clock domain.
	signal soft_trig_rx			: std_logic;	-- Single period software trigger reclocked in the RX clock domain.
	signal soft_trig_tx			: std_logic;	-- Single period software trigger reclocked in the TX clock domain.
	signal data_item			: RX_BUS;		-- Data items that may be stuffed.	
	signal hard_trig_rx			: std_logic;	-- Hardware rising edge start trigger and falling edge stop trigger in the RX clock domain.
	signal hard_trig_tx			: std_logic;	-- Hardware rising edge start trigger and falling edge stop trigger in the TX clock domain.
	signal clk_init_rst			: std_logic;  	-- Active high reset to the ADC/DAC interface core.
	signal rx_clk				: std_logic;	-- Clock assigned to the receiver datapath.
	signal tx_clk				: std_logic;	-- Clock assigned to the transmitter datapath.
	signal clk_lck_ctrl			: std_logic_vector(1 downto 0);	-- Rising edge initiates the clock lock sequence that sets IODELAY values for data inputs.        
	signal clock_status			: std_logic_vector(1 downto 0);		-- Bus used to report clock stability status.
	signal clk200 				: std_logic;	-- 200 MHz clock on global clock buffer.
	

begin

--***************************************************** HOST BUS INTERFACE *****************************************************

    -- PCI Express interface including DMA and target transaction decoders.
    PCI : entity work.pci_express
	port map (
		sys_rst_n		=> sys_rst_n,
		sys_clk			=> sys_clk,
		pci_exp_rxp		=> pci_exp_rxp,
		pci_exp_rxn		=> pci_exp_rxn,
		pci_exp_txp		=> pci_exp_txp,
		pci_exp_txn		=> pci_exp_txn,
		user_clk		=> user_clk,
		user_reset		=> user_reset,
		slv_addr 		=> slv_addr,
		slv_dataout 	=> slv_dataout,
		slv_bytevalid 	=> slv_bytevalid,
		bar0_rddata 	=> bar0_rddata,
		bar0_write 		=> bar0_write,
		bar0_read 		=> bar0_read,
		bar2_rddata 	=> bar2_rddata,
		bar2_write 		=> bar2_write,
		bar2_read 		=> bar2_read,
		dma_wr 			=> dma_wr,
		dma_rd 			=> dma_rd,
		dma_rdaddr 		=> dma_rdaddr,
		dma_wrchannel 	=> dma_wrchannel,
		dma_rdchannel 	=> dma_rdchannel,
		dma_regin		=> dma_regin,
		dma_param		=> dma_param,
		dma_control		=> dma_control,
		dma_status		=> dma_status,
		dma_fifocnt		=> dma_fifocnt,
		dma_rddata		=> dma_rddata,
		dma_wrdata		=> dma_wrdata,
		int_request		=> int_request,
		int_timeout		=> int_timeout,
		int_response	=> int_response
	);

    -- Address decoder for base address register (BAR) zero.
    BAR0 : entity work.bar0_decoder
	port map (
	    rst 				=> rst,
		clk 				=> user_clk,
	    slv_addr 			=> slv_addr(13 downto 2),
	    slv_dataout 		=> slv_dataout,
	    slv_bytevalid 		=> slv_bytevalid,
	    bar0_write			=> bar0_write,
	    bar0_read			=> bar0_read,
	    bar0_rddata			=> bar0_rddata,
		dma_rd 				=> dma_rd,
	    dma_rdaddr        	=> dma_rdaddr(13 downto 3),
		dma_rdchannel 		=> dma_rdchannel(0),
		dma_current_page	=> dma_current_page,
		dma_current_burst	=> dma_current_burst,
		dma_page_address	=> dma_page_address,
		dma_descriptor		=> dma_descriptor,
		dma_engine			=> dma_engine,
		global_int_mask 	=> global_int_mask,
		global_status		=> global_status,
		int_timeout			=> int_timeout,
		int_response		=> int_response,
		serial_config		=> serial_config,
		serial_payload		=> serial_payload,
		serial_start		=> serial_start,
		serial_busy			=> serial_busy,
		serial_shift		=> serial_shift,
		serial_read			=> serial_read,
		clk_init_start		=> clk_init_start,
		clk_init_busy		=> clk_init_busy,
	    software_reset 		=> software_reset,
		fault_monitor 		=> fault_monitor,
		fault_status 		=> fault_status,
		fault_int_mask 		=> fault_int_mask,
		bar0_address_error	=> bar0_address_error
	);

    -- Address decoder for base address register (BAR) two.
	BAR2 : entity work.bar2_decoder 
	port map (
		rst    			  	=> rst,
		clk		          	=> user_clk,
		bar2_write 		  	=> bar2_write,
	    slv_dataout       	=> slv_dataout,
		slv_bytevalid 	  	=> slv_bytevalid,
	    slv_addr 			=> slv_addr(15 downto 2),
		bar2_read		  	=> bar2_read,
		bar2_rddata		  	=> bar2_rddata,
	    dma_rd            	=> dma_rd,
	    dma_rdaddr        	=> dma_rdaddr(15 downto 3),
		dma_rdchannel     	=> dma_rdchannel(0),
		synchronizer		=> synchronizer,
		synchronizer_on_off	=> synchronizer_on_off,
		synchronizer_status	=> synchronizer_status,
		soft_trig			=> soft_trig,
		led_control			=> led_on,
		channel_monitor 	=> channel_monitor,
		channel_status 		=> channel_status,
		channel_int_mask	=> channel_int_mask,
		bar2_address_error  => bar2_address_error
	);
	
    -- Tie off DMA engine status bits used by BAR0 for unused channels.
	NO_CH : if TX_CHANNEL_COUNT+RX_CHANNEL_COUNT < 4 generate
		NO_DMA : for i in TX_CHANNEL_COUNT+RX_CHANNEL_COUNT+1 to 4 generate
			dma_engine(i) <= (others => '1');
		end generate;
	end generate;

--*********************************************** CLOCK/RESET SIGNAL ASSIGNMENTS ***********************************************

    -- Clock assignments.
    CLK_SYNC : entity work.clock_sync 
	port map (
		rst				=> rst,
		sample_clk_in	=> sample_clk_in,
		rx_clk_in  		=> rx_clk_in,
		tx_clk_in   	=> tx_clk_in,
		clk200_in		=> clk200_in,
		rx_clk 			=> rx_clk,
		tx_clk			=> tx_clk,
		clk200			=> clk200
	);
        
    -- Generate a global reset from the PCI Express reset or software reset.
	rst <= user_reset or software_reset;
	
--******************************************************** RX CHANNELS *********************************************************

	-- Instantiate the requested number of receiver channels.
	RX_CH : if RX_CHANNEL_COUNT > 0 generate
		-- Interface to the ADC chips that source data to each receiver channel.  
		ADC : entity work.adc_interface
		port map (
			rst         	=> clk_init_rst,
			clk				=> user_clk,
			rx_clk			=> rx_clk,
			clk200 			=> clk200,
			rx_data_out		=> rx_data_out,
			rx_ovr_out		=> rx_ovr_out,
			clk_lck_start   => clk_lck_ctrl(0),
			clk_lck_done   	=> clk_lck_ctrl(1),
			rx_data_in		=> rx_data_in,
			rx_ovr_in		=> rx_ovr_in,
			rx_clk_in  		=> rx_clk_in
		);
		-- Reclock the hardware trigger in the RX clock domain.
		process (rx_clk)
		begin
			if rising_edge(rx_clk) then
        		-- Allow hardware triggering from either the coax or GPIO trigger inputs.
        		hard_trig_rx <= coax_trig or gpio_trig;
			end if;
		end process;
        -- Reclock the single clock period software trigger in the RX clock domain.
		RX_TRIG : entity work.clock_crossing
		port map (
			rst					=> rst,
			source_clk			=> user_clk,
			target_clk  		=> rx_clk,
			source_to_target	=> soft_trig,
			target_enable		=> soft_trig_rx
		);
		-- Generate a datapath for the number of receiver channels requested.
		RX : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate
			-- Instantiate each receiver datapath.
			RX_DATA : entity work.rx_datapath
			port map (
				rst					=> rst,
				user_clk			=> user_clk,
				rx_clk				=> rx_clk,
				data_item			=> rx_data_out(i),
				synchronizer_on_off	=> synchronizer_on_off(i),
				synchronizer_status	=> synchronizer_status(i),
				start_event			=> synchronizer(i)(1 downto 0),
				stop_event			=> synchronizer(i)(3 downto 2),
				sample_count		=> synchronizer(i)(35 downto 4),
				soft_trigger		=> soft_trig_rx,
				hard_trigger		=> hard_trig_rx,
				dma_page_address	=> dma_page_address(i),
				dma_descriptor		=> dma_descriptor(i),
				dma_current_page	=> dma_current_page(i),
				dma_current_burst	=> dma_current_burst(i),
				dma_rddata			=> dma_rddata(i),
				dma_marker			=> dma_marker(i),
				fifo_error			=> fifo_error(i),
				dma_wr				=> dma_wr,
				dma_wrchannel		=> dma_wrchannel(i),
				dma_rd				=> dma_rd,
				dma_rdchannel		=> dma_rdchannel(i),
				dma_regin			=> dma_regin(i),
				dma_param			=> dma_param(i),
				dma_control			=> dma_control(i),
				dma_status			=> dma_status(i)(3),
				dma_engine			=> dma_engine(i),
				dma_fifocnt			=> dma_fifocnt(i)
			);
			-- Assign over-range bits to the error register assigned to the datapath.
			rx_tx_error(i) <= '0' when to_integer(unsigned(rx_ovr_out(i))) = 0 else '1';
		end generate;
	end generate;

	-- Assign unused signals if there are no RX channels.
	NO_RX : if RX_CHANNEL_COUNT = 0 generate
		clk_lck_ctrl(1) <= clk_lck_ctrl(0);
		tx_clk_out(RX_CHANNEL_COUNT+1) <= '0';
	end generate;
	
--******************************************************** TX CHANNELS *********************************************************

	-- Instantiate the requested number of transmitter channels.
	TX_CH : if TX_CHANNEL_COUNT > 0 generate
		-- Interface to the DAC chips that consume data from each transmitter channel.  
		DAC : entity work.dac_interface
		port map (
			rst         	=> clk_init_rst,
			tx_clk			=> tx_clk,
			tx_data_in		=> tx_data_in,
			tx_data_out		=> tx_data_out,
			tx_clk_out		=> tx_clk_out
		);
		-- Reclock the hardware trigger in the RX clock domain.
		process (tx_clk)
		begin
			if rising_edge (tx_clk) then
        		-- Allow hardware triggering from either the coax or GPIO trigger inputs.
        		hard_trig_tx <= coax_trig or gpio_trig;
			end if;
		end process;
        -- Reclock the single clock period software trigger in the TX clock domain.
		TX_TRIG : entity work.clock_crossing
		port map (
			rst					=> rst,
			source_clk			=> user_clk,
			target_clk  		=> tx_clk,
			source_to_target	=> soft_trig,
			target_enable		=> soft_trig_tx
		);
		-- Generate a datapath for the number of transmitter channels requested.
		TX : for i in RX_CHANNEL_COUNT+1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
			-- Instantiate each transmitter datapath.
			TX_DATA : entity work.tx_datapath
			port map (
				rst					=> rst,
				user_clk			=> user_clk,
				tx_clk				=> tx_clk,
				data_item			=> tx_data_in(i),
				synchronizer_on_off	=> synchronizer_on_off(i),
				synchronizer_status	=> synchronizer_status(i),
				start_event			=> synchronizer(i)(1 downto 0),
				stop_event			=> synchronizer(i)(3 downto 2),
				sample_count		=> synchronizer(i)(35 downto 4),
				soft_trigger		=> soft_trig_tx,
				hard_trigger		=> hard_trig_tx,
				dma_page_address	=> dma_page_address(i),
				dma_descriptor		=> dma_descriptor(i),
				dma_current_page	=> dma_current_page(i),
				dma_current_burst	=> dma_current_burst(i),
				dma_wrdata			=> dma_wrdata,
				dma_marker			=> dma_marker(i),
				fifo_error			=> fifo_error(i),
				dma_wr				=> dma_wr,
				dma_wrchannel		=> dma_wrchannel(i),
				dma_rd				=> dma_rd,
				dma_rdchannel		=> dma_rdchannel(i),
				dma_regin			=> dma_regin(i),
				dma_param			=> dma_param(i),
				dma_control			=> dma_control(i),
				dma_status			=> dma_status(i)(3),
				dma_engine			=> dma_engine(i),
				dma_fifocnt			=> dma_fifocnt(i)
			);
			-- Assign over-range bits to the error register assigned to the datapath.
			rx_tx_error(i) <= '0';		
		end generate;
	end generate;
	
	-- Assign unused signals if there are no TX channels.
	NO_TX : if TX_CHANNEL_COUNT = 0 generate
		tx_data_out(RX_CHANNEL_COUNT+1) <= (others => '0');
		tx_clk_out(RX_CHANNEL_COUNT+1) <= '0';
	end generate;
	
--******************************************************** CONTROLLERS *********************************************************

    -- Serial bus controller.
    SERIAL : entity work.serial_bus 
	port map (		
		rst    			=> rst,
		clk		        => user_clk,
		serial_payload	=> serial_payload,
		serial_start	=> serial_start,
		serial_busy		=> serial_busy,
		serial_shift	=> serial_shift,
		serial_read		=> serial_read,
		payload_size	=> serial_config(2 downto 0),
		active_edge		=> serial_config(3),
		bus_select		=> serial_config(6 downto 4),
		i2c_scl 		=> i2c_scl,
		i2c_sda_in 		=> i2c_sda_in,
		i2c_sda_out 	=> i2c_sda_out,
		i2c_direction 	=> i2c_direction,
	    spi_csb_cmd		=> spi_csb_cmd,
	    spi_clk			=> spi_clk,
	    spi_sdi			=> spi_sdi, 
	    spi_sdo			=> spi_sdo,
		synth_cmd		=> synth_cmd,
		gpio_user_cmd	=> gpio_user_cmd
	);

	-- Controller to initialize all clocks.
	CLK_INIT : entity work.clock_init 
	port map (		
		rst				=> rst,
		clk				=> user_clk,
		clk200			=> clk200,
		clk_init_start  => clk_init_start,
		clk_init_busy   => clk_init_busy,
		clk_init_rdy	=> clock_status(1),
		clk_init_rst	=> clk_init_rst,
		clock_status	=> clock_status(0),
		clk_lck_start	=> clk_lck_ctrl(0),
		clk_lck_done	=> clk_lck_ctrl(1)
	);
			
    -- Controller to generate optional interrupts triggered by various error and status conditions.
	INTERRUPT : entity work.interrupt_controller
	port map (
		rst 				=> rst,
		clk		        	=> user_clk,
		rx_clk 				=> rx_clk,
		tx_clk 				=> tx_clk,
		int_request 		=> int_request,
		fault_status 		=> fault_status,
		fault_int_mask 		=> fault_int_mask,
		fault_monitor 		=> fault_monitor,
		channel_status 		=> channel_status,
		channel_int_mask 	=> channel_int_mask,
		channel_monitor 	=> channel_monitor,
		bar0_address_error 	=> bar0_address_error,
		bar2_address_error 	=> bar2_address_error,
		fifo_error 			=> fifo_error,
		rx_tx_error 		=> rx_tx_error,
		clock_status		=> clock_status,
		dma_marker 			=> dma_marker,
		global_int_mask 	=> global_int_mask,
		global_status		=> global_status
	);		
    
	--  Instantiate the IDELAYCTRL primitive.
	IDELAY : IDELAYCTRL
		port map (RDY => clock_status(0), REFCLK => clk200, RST => clk_init_rst);	       
		
--************************************************ GPIO/USER SIGNAL ASSIGNMENTS ************************************************
	
    -- The GPIO port is not used in this demostration, so the output signals are tied off below.
    -- Tristate the GPIO voltage translators.
	gpio_tristate <= '1';
    -- Set the GPIO direction to inputs.
	gpio_direction <= (others => '1');
    -- Tie the GPIO outputs low.
	gpio_out <= user_in(6 downto 1);
	
    -- The user port is not used in this demostration, so the output signals are tied off below.
    -- Tristate the user voltage translators.
	user_tristate <= '0';
    -- Set the user direction to inputs.
	user_direction <= (others => '1');
    -- Tie the user outputs low.
	user_out <= (others => '0');
	
end structural;